Class: DaVinciCRDTestKit::ServerAbstractInvokeHookTest
- Inherits:
-
Inferno::Test
- Object
- Inferno::Test
- DaVinciCRDTestKit::ServerAbstractInvokeHookTest
show all
- Includes:
- ServerHookHelper
- Defined in:
- lib/davinci_crd_test_kit/server/server_abstract_invoke_hook_test.rb
Instance Method Summary
collapse
#discovered_service_id_for_hook, #identify_hook, #target_service_id, #tested_hook_name
Instance Method Details
#check_request_length(payloads) ⇒ Object
154
155
156
157
|
# File 'lib/davinci_crd_test_kit/server/server_abstract_invoke_hook_test.rb', line 154
def check_request_length(payloads)
skip_if tested_hook_name == ANY_HOOK_TAG && payloads.length != 1,
'The *Demonstrate a Hook Invocation* test supports only one request body.'
end
|
#continuation_message(continuation_url) ⇒ Object
173
174
175
176
177
|
# File 'lib/davinci_crd_test_kit/server/server_abstract_invoke_hook_test.rb', line 173
def continuation_message(continuation_url)
'Once all hook requests have been made and the server has gathered all desired information ' \
"from the FHIR server of Inferno's simulated CRD Client, [click here](#{continuation_url}) " \
'to continue the tests.'
end
|
#coverage_info_configuration_supported? ⇒ Boolean
179
180
181
|
# File 'lib/davinci_crd_test_kit/server/server_abstract_invoke_hook_test.rb', line 179
def coverage_info_configuration_supported?
false
end
|
159
160
161
|
# File 'lib/davinci_crd_test_kit/server/server_abstract_invoke_hook_test.rb', line 159
def perform_invoke_hook_job(*)
Inferno::Jobs.perform(DaVinciCRDTestKit::Jobs::InvokeHook, *)
end
|
#wait_message(acknowledge_before_continuing, continuation_url) ⇒ Object
163
164
165
166
167
168
169
170
171
|
# File 'lib/davinci_crd_test_kit/server/server_abstract_invoke_hook_test.rb', line 163
def wait_message(acknowledge_before_continuing, continuation_url)
message = "Inferno's simulated CRD Client will initiate hook requests to the server. " \
'During these invocations the server will be able to make FHIR requests ' \
"against Inferno's simulated CRD Client."
return message unless acknowledge_before_continuing
"#{message}\n\n#{continuation_message(continuation_url)}"
end
|