Class: DaVinciCRDTestKit::V221::ClientFHIRAPIGroup

Inherits:
Inferno::TestGroup
  • Object
show all
Defined in:
lib/davinci_crd_test_kit/client/v2.2.1/client_fhir_api_group.rb

Constant Summary collapse

INPUT_INSTRUCTIONS =
%(
  The location of and an access token for the client's FHIR server are expected to come in
  the CDS Hooks request. The following information will be auto-populated from
  the body of the latest hook invocation made during this session:
  - The URL of the CRD client FHIR server from the `fhirServer` field.
  - The OAuth Access Token from the `fhirAuthorization.access_token` field.

  If that access token will not be long-lived enough to perform
  a full test of the US Core FHIR API, provide an updated token and/or refresh
  token with Client ID and token endpoint so that Inferno can perform these tests.
).freeze