Class: DaVinciCRDTestKit::ServerAbstractInvokeHookTest

Inherits:
Inferno::Test
  • Object
show all
Includes:
ServerHookHelper
Defined in:
lib/davinci_crd_test_kit/server/server_abstract_invoke_hook_test.rb

Instance Method Summary collapse

Methods included from ServerHookHelper

#discovered_service_id_for_hook, #identify_hook, #target_service_id, #tested_hook_name

Instance Method Details

#continuation_message(continuation_url) ⇒ Object



155
156
157
158
159
# File 'lib/davinci_crd_test_kit/server/server_abstract_invoke_hook_test.rb', line 155

def continuation_message(continuation_url)
  'Once all hook requests have been made and the server has gathered all desired information ' \
    "from the FHIR server of Inferno's simulated CRD Client, [click here](#{continuation_url}) " \
    'to continue the tests.'
end

#coverage_info_configuration_supported?Boolean

Returns:

  • (Boolean)


161
162
163
# File 'lib/davinci_crd_test_kit/server/server_abstract_invoke_hook_test.rb', line 161

def coverage_info_configuration_supported?
  false
end

#wait_message(acknowledge_before_continuing, continuation_url) ⇒ Object



145
146
147
148
149
150
151
152
153
# File 'lib/davinci_crd_test_kit/server/server_abstract_invoke_hook_test.rb', line 145

def wait_message(acknowledge_before_continuing, continuation_url)
  message = "Inferno's simulated CRD Client will initiate hook requests to the server. " \
            'During these invocations the server will be able to make FHIR requests ' \
            "against Inferno's simulated CRD Client."

  return message unless acknowledge_before_continuing

  "#{message}\n\n#{continuation_message(continuation_url)}"
end